# coding: utf8

"""
This module includes the `Show` class and the main functionalities of `auditorium`.
"""

import asyncio
import base64
import io
import json
import runpy
import warnings
import webbrowser
from collections import OrderedDict
from typing import Union

import websockets
from fastapi import FastAPI
from jinja2 import Template
from markdown import markdown
from pydantic import BaseModel
from pygments import highlight
from pygments.formatters.html import HtmlFormatter
from pygments.lexers import get_lexer_by_name
from pygments.styles import get_style_by_name
from starlette.responses import HTMLResponse
from starlette.staticfiles import StaticFiles

from .components import Animation, Block, Column, Fragment, ShowMode
from .utils import fix_indent, path


class UpdateData(BaseModel):
    type: str
    id: str
    slide: str
    value: Union[int, str]


class Show(FastAPI):
    def __init__(self, title="", theme="white", code_style="monokai"):
        self.theme = theme
        self.formatter = HtmlFormatter(style=get_style_by_name(code_style))

        self._slides = {}
        self._sections = []
        self._tail = []

        self.app = FastAPI()
        self.app.get("/")(self._index)
        self.app.post("/update")(self._update)
        self.app.mount("/static", StaticFiles(directory=path("static")))

        self._title = title
        self._current_section = None
        self._rendering = False

        with open(path("templates/content.html")) as fp:
            self._template_content = Template(fp.read())

        with open(path("templates/static.html")) as fp:
            self._template_static = Template(fp.read())

        with open(path("templates/dynamic.html")) as fp:
            self._template_dynamic = Template(fp.read())

    ## Show functions

    def run(self, *, host: str, port: int, debug: bool = False) -> None:
        self._content = self._render_content()

        try:
            import uvicorn

            uvicorn.run(self.app, host=host, port=port, debug=debug)
        except ImportError:
            warnings.warn("(!) You need `uvicorn` installed in order to call `run`.")
            exit(1)

    def publish(self, server: str, name: str):
        url = "{}/ws".format(server)
        asyncio.get_event_loop().run_until_complete(self._ws(url, name))

    async def _ws(self, url: str, name: str):
        try:
            async with websockets.connect(url) as websocket:
                print("Connected to server")
                await websocket.send(name)
                print("Starting command loop.")

                while True:
                    command = await websocket.recv()
                    command = json.loads(command)

                    response = self._do_ws_command(command)
                    response = json.dumps(response)
                    await websocket.send(response)
        except ConnectionRefusedError:
            print("(!) Could not connect to %s. Make sure server is up." % url)
            exit(1)
        except websockets.exceptions.ConnectionClosedError:
            print("(!) Connection to %s closed by server." % url)
            exit(1)


    def _do_ws_command(self, command):
        if command["type"] == "render":
            print("Rendering content")
            return dict(content=self.render())
        if command["type"] == "ping":
            print("Saying hello")
            return dict(msg="pong")
        elif command["type"] == "error":
            print("(!) %s" % command['msg'])
            raise websockets.exceptions.ConnectionClosedError(1006, command['msg'])
        else:
            print("Executing slide %s" % command["slide"])
            values = {}
            values[command["id"]] = command["value"]
            update = self.do_code(command["slide"], values)
            return update

        raise ValueError("Unknown command: %s", command["type"])

    @property
    def show_title(self) -> str:
        return self._title

    def append(self, show, instance_name="show"):
        if isinstance(show, str):
            show = Show.load(show, instance_name)

        self._tail.append(show)
        return self

    @staticmethod
    def load(path, instance_name="show"):
        from .markdown import MarkdownLoader

        if path.endswith(".py"):
            ns = runpy.run_path(path)
            show = ns[instance_name]
        elif path.endswith(".md"):
            loader = MarkdownLoader(path, instance_name=instance_name)
            show = loader.parse()
        else:
            raise ValueError("Invalid path %r" % path)

        return show

    def get_slide(self, slide_id):
        if slide_id in self._slides:
            return self._slides[slide_id]

        for show in self._tail:
            try:
                return show.get_slide(slide_id)
            except:
                pass

        raise ValueError(f"Invalid slide id: {slide_id}")

    @property
    def slides(self):
        yield from self._slides

        for show in self._tail:
            yield from show.slides

    @property
    def sections(self):
        yield from self._sections

        for show in self._tail:
            yield from show.sections

    ## @slide decorator

    def slide(self, func=None, id=None):
        if func is not None:
            return self._wrap(func, id)

        elif id is not None:

            def wrapper(func):
                return self._wrap(func, id)

            return wrapper

    @staticmethod
    def _vertical_slide_wrapper(show, section):
        class _VerticalWrapper:
            def __init__(self, section):
                self.section = section

            def slide(self, func=None, id=None):
                if func is not None:
                    return show._wrap(func, id, self.section)

                elif id is not None:

                    def wrapper(func):
                        return show._wrap(func, id, self.section)

                    return wrapper

        return _VerticalWrapper(section)

    def _wrap(self, func, id, section=None):
        if self._rendering:
            return func

        slide_id = id or func.__name__
        slide = Slide(slide_id, func, self)
        self._slides[slide_id] = slide

        if section is None:
            section = Section()
            section.add_slide(slide)
            self._sections.append(section)
            wrapper = Show._vertical_slide_wrapper(self, section)
            func.slide = wrapper.slide
        else:
            section.add_slide(slide)

        return func

    ## Internal API

    def do_markup(self, slide):
        slide = self.get_slide(slide)
        ctx = slide.run(ShowMode.Markup)
        return "\n\n".join(ctx.content)

    def do_code(self, slide, values):
        slide = self.get_slide(slide)
        ctx = slide.run(ShowMode.Code, values)
        return ctx.update

    def render(self, theme=None):
        return self._template_static.render(
            show=self,
            content=self._render_content(),
            code_style=self._code_style(),
            embed=self._embed,
            theme=theme or self.theme,
        )

    ## Utilities

    def _render_content(self):
        self._rendering = True
        return self._template_content.render(show=self)

    def _embed(self, src):
        with open(path(src)) as fp:
            return fp.read()

    def _code_style(self):
        return self.formatter.get_style_defs(".highlight")

    ## Routes

    async def _update(self, data: UpdateData):
        values = {}
        values[data.id] = data.value
        update = self.do_code(data.slide, values)
        return update

    async def _index(self, theme: str = "white"):
        return HTMLResponse(
            self._template_dynamic.render(
                show=self,
                content=self._content,
                code_style=self._code_style(),
                theme=theme,
            )
        )


class Context:
    def __init__(self, slide_id, mode, show, values=None):
        self.content = []
        self.update = {}
        self.slide_id = slide_id
        self.unique_id = 0
        self.mode = mode
        self.values = values
        self.show = show

    ## Utility methods

    def _get_unique_id(self, markup):
        self.unique_id += 1
        item_id = f"{self.slide_id}-{markup}-{self.unique_id - 1}"
        return item_id, f'id="{item_id}" data-slide="{self.slide_id}"'

    ## Binding methods

    def title(self, text):
        return self.header(text, 1)

    def header(self, text, level=2):
        return self.markdown("#" * level + " " + text)

    def hrule(self):
        return self.markup("<hr>")

    def markup(self, content):
        item_id, id_markup = self._get_unique_id("markup")

        if self.mode == ShowMode.Markup:
            self.content.append(f"<div {id_markup}>{fix_indent(content)}</div>")
        else:
            self.update[item_id] = fix_indent(content)

    def markdown(self, content):
        item_id, id_markup = self._get_unique_id("markdown")

        if self.mode == ShowMode.Markup:
            self.content.append(
                f"<div {id_markup}>{markdown(fix_indent(content))}</div>"
            )
        else:
            self.update[item_id] = markdown(fix_indent(content))

    def text_input(self, default="", track_keys=True):
        item_id, id_markup = self._get_unique_id("text-input")

        event = "onkeyup" if track_keys else "onchange"

        if self.mode == ShowMode.Markup:
            self.content.append(
                f'<input {id_markup} data-event="{event}" type="text" class="text" value="{default}"></input>'
            )
            return default
        else:
            return self.values[item_id]

    def pyplot(self, plt, height=400, aspect=4 / 3, fmt="png"):
        item_id, id_markup = self._get_unique_id("pyplot")
        plt.tight_layout()
        buffer = io.BytesIO()
        plt.savefig(buffer, format=fmt)
        plt.clf()
        buffer.seek(0)

        if fmt == "png":
            code = base64.b64encode(buffer.read(-1)).decode("ascii")
            markup = f'<img width="{aspect * height}px" height="{height}px" class="pyplot" src="data:image/png;base64, {code}"></img>'
        elif fmt == "svg":
            code = buffer.read(-1).decode("ascii")
            markup = code
        else:
            raise ValueError(
                "Invalid value for parameter `fmt`, must be 'png' or 'svg'."
            )

        if self.mode == ShowMode.Markup:
            self.content.append(f"<div {id_markup}>{markup}</div>")
        else:
            self.update[item_id] = markup

    def anchor(self, slide_or_href, text=None):
        item_id, id_markup = self._get_unique_id("anchor")

        if self.mode == ShowMode.Markup:
            if hasattr(slide_or_href, "__name__"):
                slide_or_href = "#/" + slide_or_href.__name__

            if text is None:
                text = slide_or_href

            self.content.append(f'<a {id_markup} href="{slide_or_href}">{text}</a>')
        else:
            self.update[item_id] = text

    def code(self, text, language="python"):
        item_id, id_markup = self._get_unique_id("code")
        content = fix_indent(text)

        lexer = get_lexer_by_name(language)
        code = highlight(content, lexer, self.show.formatter)

        if self.mode == ShowMode.Markup:
            self.content.append(f"<div {id_markup}>{code}</div>")
        else:
            self.update[item_id] = code

    def animation(self, steps=10, time=0.1, loop=True) -> Animation:
        item_id, id_markup = self._get_unique_id("animation")

        if self.mode == ShowMode.Markup:
            self.content.append(
                f'<animation {id_markup} data-steps="{steps}" data-time="{time}" data-loop="{loop}"></animation>'
            )
            return Animation(steps, time, loop, 0)

        return Animation(steps, time, loop, self.values[item_id])

    def columns(self, *widths) -> Column:
        return Column(self, *widths)

    def fragment(self, style="fade-in") -> Fragment:
        return Fragment(self, style)

    def block(self, title="", style="default") -> Block:
        return Block(self, title, style)

    def success(self, title="") -> Block:
        return self.block(title, "success")

    def warning(self, title="") -> Block:
        return self.block(title, "warning")

    def error(self, title="") -> Block:
        return self.block(title, "error")


class Section:
    def __init__(self):
        self._slides = []
        self._slide_ids = {}

    def add_slide(self, slide):
        if slide.slide_id in self._slide_ids:
            raise ValueError("Duplicated slide name: %r" % slide.slide_id)

        self._slides.append(slide)
        self._slide_ids[slide.slide_id] = slide

    @property
    def slides(self):
        for slide in self._slides:
            yield slide.slide_id


class Slide:
    def __init__(self, slide_id: str, func, show):
        self._slide_id = slide_id
        self._func = func
        self._show = show

    @property
    def slide_id(self) -> str:
        return self._slide_id

    def run(self, mode, values=None) -> Context:
        ctx = Context(self.slide_id, mode, self._show, values)

        if self._func.__doc__:
            ctx.markdown(self._func.__doc__)

        self._func(ctx)
        return ctx



# coding: utf8

import fire
import runpy
import webbrowser

from auditorium import Show


class Auditorium:
    @staticmethod
    def run(
        path,
        *,
        host: str = "127.0.0.1",
        port: int = 6789,
        debug: bool = False,
        instance_name: str = "show",
    ):
        "Runs a custom Python script as a slideshow."

        show = Show.load(path, instance_name)
        show.run(host=host, port=port, debug=debug)

    @staticmethod
    def publish(
        path: str,
        name: str,
        *,
        server: str = "ws://auditorium.apiad.net",
        instance_name: str = "show"
    ):
        show = Show.load(path, instance_name)
        show.publish(server=server, name=name)

    @staticmethod
    def demo(host: str = "127.0.0.1", port: int = 6789, debug: bool = False):
        "Starts the demo slideshow."

        from auditorium.demo import show

        show.run(host=host, port=port, debug=debug)

    @staticmethod
    def render(path, theme="white", instance_name="show"):
        "Renders a slideshow into a single HTML with all resources embedded."

        show = Show.load(path, instance_name)
        print(show.render(theme))

    @staticmethod
    def server(host: str = "0.0.0.0", port: int = 9876):
        from auditorium.server import run_server

        run_server(host=host, port=port)

    @staticmethod
    def test():
        return "It's OK!"


def main():
    fire.Fire(Auditorium, name="auditorium")


if __name__ == "__main__":
    main()



# coding: utf8

import warnings
import websockets

from typing import Dict, Tuple
from fastapi import FastAPI, HTTPException
from starlette.responses import HTMLResponse
from starlette.websockets import WebSocket
from starlette.staticfiles import StaticFiles
import asyncio
from jinja2 import Template

from .utils import path
from .show import UpdateData

server = FastAPI()
server.mount("/static", StaticFiles(directory=path("static")))

SERVERS: Dict[str, Tuple[asyncio.Queue, asyncio.Queue]] = {}


@server.get("/")
async def index():
    with open(path('templates/server.html')) as fp:
        TEMPLATE = Template(fp.read())

    return HTMLResponse(TEMPLATE.render(servers_list=list(SERVERS)))


@server.get("/{name}/")
async def render(name: str):
    try:
        queue_in, queue_out = SERVERS[name]
    except KeyError:
        raise HTTPException(404)

    await queue_in.put(dict(type="render"))

    response = await queue_out.get()
    queue_out.task_done()

    return HTMLResponse(response["content"])


@server.post("/{name}/update")
async def update(name: str, data: UpdateData):
    try:
        queue_in, queue_out = SERVERS[name]
    except KeyError:
        raise HTTPException(404)

    await queue_in.put(data.dict())

    response = await queue_out.get()
    queue_out.task_done()

    return response


async def ping(name):
    try:
        queue_in, queue_out = SERVERS[name]
        await queue_in.put(dict(type="ping"))
        response = await queue_out.get()
        assert response['msg'] == 'pong'
        return True
    except:
        return False


@server.websocket("/ws")
async def ws(websocket: WebSocket):
    await websocket.accept()
    name = await websocket.receive_text()

    if name in SERVERS:
        alive = await ping(name)

        if alive:
            await websocket.send_json(dict(type="error", msg="Name is already taken."))
            await websocket.close()
            return

    print("Registering new server: ", name)

    queue_in: asyncio.Queue = asyncio.Queue()
    queue_out: asyncio.Queue = asyncio.Queue()

    SERVERS[name] = (queue_in, queue_out)

    try:
        while True:
            command = await queue_in.get()
            await websocket.send_json(command)
            response = await websocket.receive_json()

            queue_in.task_done()
            await queue_out.put(response)
    except:
        print("(!) Connection to %s closed by client." % name)

        for _ in range(queue_in.qsize()):
            queue_in.task_done()

        for _ in range(queue_out.qsize()):
            queue_out.task_done()

    print("Unregistering server:", name)
    SERVERS.pop(name)


def run_server(*, host="0.0.0.0", port=9876):
    try:
        import uvicorn

        uvicorn.run(server, host=host, port=port)
    except ImportError:
        warnings.warn("(!) You need `uvicorn` installed in order to call `server`.")
        exit(1)



# coding: utf8

from auditorium import Show
from typing import List
from auditorium.utils import path


def load_markdown_demo():
    return Show.load(path("./static/md/demo.md"))


class MarkdownLoader:
    def __init__(self, path, instance_name="ctx"):
        self.path = path
        self.instance_name = instance_name

    def parse(self):
        slides = []
        current_slide = []

        with open(self.path) as fp:
            for line in fp:
                line = line.strip("\n")

                if line.startswith("## ") and current_slide:
                    slides.append(current_slide)
                    current_slide = []

                current_slide.append(line)

            if current_slide:
                slides.append(current_slide)

        show = Show()

        for i, slide in enumerate(slides):
            show.slide(func=MarkdownSlide(show, slide), id="slide-%i" % (i + 1))

        return show


class MarkdownSlide:
    def __init__(self, show: Show, content):
        self.show = show
        self.content: List[Content] = []

        state = "markdown"  # or 'code'
        language = ""
        code_start = ""
        split: List[str] = []

        for line in content:
            if state == "markdown":
                if line.startswith("```") or line.startswith("~~~"):
                    if split:
                        self.content.append(MarkdownContent(split))

                    split = []
                    state = "code"
                    code_start = line[:3]
                    language = line[3:].split()[0]
                    tags = line[3:].split()[1:]
                else:
                    split.append(line)

            elif state == "code":
                if line == code_start:
                    if split:
                        self.content.append(CodeContent(split, language, tags))

                    split = []
                    state = "markdown"
                else:
                    split.append(line)

        if split:
            if state == "markdown":
                self.content.append(MarkdownContent(split))
            else:
                raise ValueError("Didn't closed a code line...")

    def __call__(self, ctx):
        global_context = dict(ctx=ctx)

        for content in self.content:
            content(ctx, global_context)


class Content:
    def __call__(self, show, global_context):
        self._call(show, global_context)


class MarkdownContent(Content):
    def __init__(self, lines):
        self.lines = "\n".join(lines)

    def _call(self, show, global_context):
        show.markdown(self.lines.format(**global_context))


class CodeContent(Content):
    def __init__(self, lines, language, tags):
        self.lines = "\n".join(lines)
        self.tags = tags
        self.language = language

    def _call(self, show, global_context):
        run = ":run" in self.tags
        echo = not run or ":echo" in self.tags
        persist = ":persist" in self.tags

        local_context = dict()

        if run:
            exec(self.lines, global_context, local_context)

        if echo:
            show.code(self.lines, self.language)

        if persist:
            global_context.update(local_context)



# coding: utf8

"""
A simple demo script for `auditorium`.
"""

from auditorium import Show

show = Show("Auditorium Demo")
app = show.app


@show.slide
def intro(ctx):
    """
    # Auditorium
    ## A Python-powered slideshow generator with steroids
    """


@show.slide
def what_is_this(ctx):
    """
    ## What is Auditorium

    `auditorium` is a Python module for creating slideshows which
    are ultimately displayed in a browser using the amazing
    library `reveal.js`.

    With `auditorium` you don't need to learn JavaScript,
    HTML or CSS. Everything goes in your Python code,
    both presentation content and logic.
    """


@show.slide
def how_it_works(ctx):
    """
    ## How it works

    `auditorium` creates a continuous feedback loop between Python
    and HTML/JavaScript.

    * Every slide in `auditorium` is a Python method.
    * You can mix up content with logic.
        - Inject variables into the presentation.
        - Receive back interaction events and values.
    * Layout is handled by `reveal.js`.
    * You only ever need to write Python code.
    """


@show.slide
def the_basics(ctx):
    ctx.markdown("## The basics")
    ctx.markdown("Start by declaring a `Show` instance.")

    ctx.code(
        """
        from auditorium import Show
        show = Show("Auditorium Demo")
        """
    )

    ctx.markdown("Add a method for every slide, decorated with `@show.slide`.")

    ctx.code(
        """
        @show.slide
        def my_slide(ctx):
            # slide content
        """
    )

    ctx.markdown("Finally run the show.")

    ctx.code("auditorium run [file.py]", "bash")


@show.slide
def examples(ctx):
    ctx.header("Some examples")
    ctx.markdown("Slide content is added with _markdown_.")
    ctx.markup("Or <strong>directly</strong> as HTML.")

    ctx.hrule()

    ctx.code(
        """
        @show.slide
        def example(ctx):
            ctx.header("Some examples")
            ctx.markdown("Slide content is added with _markdown_.")
            ctx.markup("Or <strong>directly</strong> as HTML.")
        """
    )


@show.slide
def static_content(ctx):
    """
    ## Static Content

    Static content can also be added directly as markdown in
    the `docstring` of the corresponding method.
    """

    ctx.hrule()

    ctx.code(
        """
        @show.slide
        def static_content(ctx):
            \"\"\"
            ## Static Content

            Static content can also be added directly as markdown in
            the `docstring` of the corresponding method.
            \"\"\"
        """
    )


@show.slide
def layout(ctx):
    """
    ## Custom layout

    More complex layouts can be composed.
    """

    ctx.hrule()

    with ctx.columns(2, 3) as cl:
        ctx.markdown("#### Content")
        ctx.markdown("You can put custom content in any column...")

        cl.tab()
        ctx.code(
            """
            with ctx.columns(2, 3) as cl:
                ctx.markdown("#### Content")
                ctx.markdown("...")

                cl.tab()
                ctx.code("...")
            """
        )


@show.slide
def data_binding(ctx):
    """
    ## Dynamic Data

    Use dynamic data with two-way binding to add executable Python logic
    to your presentation.
    """

    with ctx.columns(2) as cl:
        text = ctx.text_input("dlrow")
        text = "".join(reversed(text)).title()
        cl.tab()
        ctx.markdown(f"Hello {text}!!")

    ctx.hrule()

    ctx.code(
        """
        @show.slide
        def data_binding(ctx):
            # ...
            text = ctx.text_input("dlrow")
            text = "".join(reversed(text)).title()
            ctx.markdown(f"Hello {text}!!")
        """
    )


@show.slide
def animation(ctx):
    """
    ## Animations

    You can create simple stateless animations with pure Python
    """

    with ctx.animation(steps=10, time=0.33, loop=True) as anim:
        ctx.markdown("> ### ." + ("." * anim.current))

    ctx.hrule()

    ctx.code(
        """
        with ctx.animation(steps=10, time=0.33, loop=True) as anim:
            ctx.markdown("> ### ." + ("." * anim.current))
        """
    )


@show.slide
def vertical_slides(ctx):
    """
    ## Vertical Slides

    Vertical slides allow you to create "read-more" like
    features in your slideshow.
    They can be skipped on shorter presentations and left
    for more interested audiences.
    """

    with ctx.warning(ctx):
        ctx.markdown("Press `DOWN` instead of `LEFT` or click the down arrow.")


@vertical_slides.slide
def vertical_code(ctx):
    """
    ## Vertical Slides: Code
    """

    ctx.code(
        """
        @show.slide
        def main_slide(ctx):
            # content of main slide

        @main_slide.slide
        def vertical_1(ctx):
            # content of first vertical slide

        @main_slide.slide
        def vertical_2(ctx):
            # content of first vertical slide
        """
    )


@vertical_slides.slide
def vertical_more(ctx):
    """
    ### Vertical Slides: More Info

    If you press `SPACE` instead of `LEFT` or `DOWN`
    the slideshow will cycle through **all** of the slides
    in order, including vertical slides.

    Show will go down and left automatically as necessary.
    """


@show.slide
def blocks(ctx):
    """
    ## Blocks

    Like beamer, pre-styled blocks are available.
    """

    with ctx.columns(2) as cl:

        with ctx.block("Standard block"):
            ctx.markdown("And its content...")

        with ctx.success("Success block"):
            ctx.markdown("For happy endings...")

        cl.tab()

        with ctx.warning("Warning block"):
            ctx.markdown("For hairy stuff...")

        with ctx.error("Error block"):
            ctx.markdown("When nothing works...")


@blocks.slide
def blocks_code(ctx):
    """
    ## Blocks: Code
    """

    ctx.code(
        """
        with ctx.block('Standard block'):
            ctx.markdown("And its content...")

        with ctx.success('Success block'):
            ctx.markdown("For happy endings...")

        with ctx.warning('Warning block'):
            ctx.markdown("For hairy stuff...")

        with ctx.error('Error block'):
            ctx.markdown("When nothing works...")
        """
    )


@show.slide
def fragments(ctx):
    """
    ## Fragments

    Fragments allow to animate elements inside a slide.
    """

    with ctx.fragment(ctx):
        ctx.markdown("The can have different animations as well...")

    with ctx.fragment(ctx):
        ctx.hrule()
        ctx.code(
            """
            with ctx.fragment(style='...'): # fade-in, grow, ...
                # content
            """
        )


@fragments.slide
def fragment_examples(ctx):
    """
    ## Fragment examples

    Here are some of the possible fragment animations.
    """

    with ctx.columns(3) as cl:
        for i, style in enumerate(
            "grow shrink fade-in fade-out fade-up fade-down fade-left \
                fade-right highlight-blue highlight-red highlight-green".split()
        ):
            if i > 0 and i % 4 == 0:
                cl.tab()

            with ctx.fragment(style):
                ctx.markdown(f"`{style}`")


@show.slide
def pyplot(ctx):
    """
    ## pyplot

    Dynamically generated graphs with `pyplot` can be added
    also very easily.
    """

    try:
        from matplotlib import pyplot as plt
        import numpy as np
    except ImportError:
        with ctx.error("Dependencies missing"):
            ctx.markdown("You need `matplotlib` installed to make this slide work. Make sure to run:")

        ctx.code("pip install matplotlib", "bash")
        return

    xg = np.random.RandomState(0)
    yg = np.random.RandomState(1)

    with ctx.columns(2) as cl:
        with ctx.animation(steps=60, time=0.5, loop=True) as anim:
            x = xg.uniform(size=anim.current * 50)
            y = yg.uniform(size=anim.current * 50)
            colors = [
                "green" if xi ** 2 + yi ** 2 < 1 else "orange" for (xi, yi) in zip(x, y)
            ]
            plt.scatter(x, y, s=3, c=colors)
            plt.ylim(0, 1)
            plt.xlim(0, 1)
            ctx.pyplot(plt, fmt="png", height=350)

            cl.tab()

            with ctx.block("Monte Carlo Sampling"):
                ctx.markdown(f"Samples: {len(x)}")
                ctx.markdown(f"Inside: {colors.count('green')}")
                ctx.markdown(
                    "**Pi = %.3f**"
                    % (4 * colors.count("green") / len(x) if len(x) > 0 else 0)
                )

@pyplot.slide
def pyplot_code(ctx):
    """### Pyplot: Code"""

    ctx.code(
        """
        xg = np.random.RandomState(0)
        yg = np.random.RandomState(1)

        with ctx.animation(steps=60, time=0.5, loop=True) as anim:
            x = xg.uniform(size=anim.current * 50)
            y = yg.uniform(size=anim.current * 50)
            colors = ['green' if xi ** 2 + yi ** 2 < 1 else 'orange'
                    for (xi, yi) in zip(x,y)]
            plt.scatter(x, y, s=3, c=colors)
            plt.ylim(0, 1)
            plt.xlim(0, 1)
            ctx.pyplot(plt, fmt='png', height=350)
        """
    )


@show.slide
def static_html(ctx):
    """
    ## Going full static

    If you only need `auditorium` for the initial rendering of the HTML,
    and have no animations or interactive code, you can use:
    """

    ctx.code("auditorium render [file] > [output.html]", "bash")

    ctx.markdown(
        """
        This will render the slideshow into a static HTML with all CSS and
        JavaScript embedded, that you can take away and reproduce in any browser,
        or host on a static file server (like Github pages).
        """
    )


@show.slide
def themes(ctx):
    """
    ## Themes
    """

    ctx.markdown("A standard `reveal.js` theme can be specified at creation time:")
    ctx.code("show = Show(theme='black')")

    ctx.markdown("Or directly as a query string arg:")

    with ctx.block(ctx):
        ctx.anchor("/?theme=black#/themes")


@show.slide
def markdown_first(ctx):
    """
    ## Markdown First

    If your Python code is short and you prefer to author directly
    in Markdown, you can do **almost everything** in a Markdown file
    and insert Python here and there.

    Run your slideshow with:
    """

    ctx.code("auditorium run [file.md]")


@show.slide
def append(ctx):
    """
    ## Appendix

    You can append one `show` instance right after another, _ad-infinitum_,
    to compose larger slideshows from shorter ones.
    """

    ctx.code(
        """
        show_1 = Show("...")
        # content
        show_2 = Show("...")
        # content
        show_1.append(show_2)
        """
    )

    ctx.markdown("It also works directly with file paths, both Python and Markdown.")
    ctx.code("show.append('auditorium/static/md/demo.md')")


from auditorium.markdown import load_markdown_demo

show.append(load_markdown_demo())



# coding: utf8

from enum import IntEnum


class ShowMode(IntEnum):
    Edit = 0
    Markup = 1
    Code = 2


class Animation:
    def __init__(self, steps, time, loop, current):
        self.steps = steps
        self.time = time
        self.loop = loop
        self._current = current

    @property
    def current(self):
        return self._current

    def __enter__(self):
        return self

    def __exit__(self, *args):
        pass


class Wrapper:
    def __init__(self, ctx):
        self.ctx = ctx
        self.begin()

    def __enter__(self):
        return self

    def __exit__(self, *args):
        return self.end()

    def begin(self):
        raise NotImplementedError()

    def end(self):
        raise NotImplementedError()


class Column(Wrapper):
    def __init__(self, ctx, *widths):
        if len(widths) == 1:
            widths = [1.0 / widths[0] for _ in range(widths[0])]

        total_width = sum(widths) + 0.01
        widths = [w / total_width for w in widths]

        self.widths = list(widths)
        super(Column, self).__init__(ctx)

    def begin(self):
        self.ctx.content.append(f'<div class="columns">')
        self.ctx.content.append(
            f'<div class="column" style="width: {self.widths[0] * 100}%;">'
        )
        self.widths.pop(0)

    def tab(self):
        self.ctx.content.append(f"</div>")
        self.ctx.content.append(
            f'<div class="column" style="width: {self.widths[0] * 100}%;">'
        )
        self.widths.pop(0)

    def end(self):
        self.ctx.content.append("</div>")
        self.ctx.content.append("</div>")


class Block(Wrapper):
    def __init__(self, ctx, title, style):
        self.title = title
        self.style = style
        super(Block, self).__init__(ctx)

    def begin(self):
        self.ctx.content.append(
            f'<div class="block block-{self.style}"><h1 class="block-title">{self.title}</h1><div class="block-content">'
        )

    def end(self):
        self.ctx.content.append("</div></div>")


class Fragment(Wrapper):
    def __init__(self, ctx, style):
        self.style = style
        super(Fragment, self).__init__(ctx)

    def begin(self):
        self.ctx.content.append(f'<div class="fragment {self.style}">')

    def end(self, *args):
        self.ctx.content.append("</div>")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['auditorium']

package_data = \
{'': ['*'],
 'auditorium': ['static/css/*',
                'static/css/print/*',
                'static/css/theme/*',
                'static/img/*',
                'static/js/*',
                'static/lib/font/league-gothic/*',
                'static/lib/font/source-sans-pro/*',
                'static/lib/js/*',
                'static/md/*',
                'static/plugin/highlight/*',
                'static/plugin/math/*',
                'static/plugin/multiplex/*',
                'static/plugin/notes-server/*',
                'static/plugin/notes/*',
                'static/plugin/print-pdf/*',
                'static/plugin/search/*',
                'static/plugin/zoom-js/*',
                'templates/*']}

install_requires = \
['aiofiles>=0.4.0,<0.5.0',
 'fastapi>=0.45.0,<0.46.0',
 'fire>=0.2.1,<0.3.0',
 'jinja2>=2.10.3,<3.0.0',
 'markdown>=3.1.1,<4.0.0',
 'pygments>=2.5.2,<3.0.0']

extras_require = \
{'server': ['uvicorn>=0.11.1,<0.12.0']}

entry_points = \
{'console_scripts': ['auditorium = auditorium.__main__:main']}

setup_kwargs = {
    'name': 'auditorium',
    'version': '19.1.6',
    'description': 'A Python-powered slideshow maker with steroids.',
    'long_description': '# Auditorium\n\n[<img alt="PyPI - License" src="https://img.shields.io/pypi/l/auditorium.svg">](https://github.com/apiad/auditorium/blob/master/LICENSE)\n[<img alt="PyPI - Python Version" src="https://img.shields.io/pypi/pyversions/auditorium.svg">](https://pypi.org/project/auditorium/)\n[<img alt="PyPI" src="https://img.shields.io/pypi/v/auditorium.svg">](https://pypi.org/project/auditorium/)\n[<img alt="Travis (.org)" src="https://img.shields.io/travis/apiad/auditorium/master.svg">](https://travis-ci.org/apiad/auditorium)\n[<img alt="Codecov" src="https://img.shields.io/codecov/c/github/apiad/auditorium.svg">](https://codecov.io/gh/apiad/auditorium)\n[<img alt="Gitter" src="https://img.shields.io/gitter/room/apiad/auditorium">](https://gitter.im/auditorium-slides/community)\n[<img alt="Demo" src="https://img.shields.io/badge/demo-browse-blueviolet"></img>](https://auditorium-demo.apiad.net)\n\n<img src="https://apiad.net/auditorium/assets/logo.png"></img>\n\n> A Python-powered slideshow creator with steroids.\n\nSee the demo at [auditorium-demo.apiad.net](https://auditorium-demo.apiad.net).\n\n## What\'s this about\n\nAuditorium is a Python-powered slideshow generator.\nYou write pure Python code, and obtain an HTML+JavaScript slideshow (using the amazing [reveal.js](https://revealjs.com) library).\nThe awesomeness comes from the fact that your Python backend is connected to the slideshow frontend, which allows your slideshow to dynamically change according to user input or automatically.\n\nThis opens the door to a few interesting use cases:\n\n*  The slides content itself can be generated by code. For example, long and repetitive sets of slides can be automatically generated, or tables and graphs embedded in slides can be generated on-the-fly with `matplotlib`, `bokeh`, `altair`, `plotly`, or any visualization package that produces HTML output.\n*  You can insert components in the slides that respond to user input, and execute a Python code in response. For example, you can generate an interactive graph that can be modified by moving sliders in a slideshow.\n*  You can create beautiful animations with simple Python code, that automatically play on a slide, using visualization libraries or simple HTML markup.\n\n> **And all of this without writing a single line of HTML or JavaScript.**\n\nAlternatively, if you need little to no Python code, you can author your slideshow in pure Markdown and add some Python sprinkless here and there when necessary.\n\n## Installation\n\nSimply run:\n\n    pip install auditorium[server]\n\nTo see a quick demo run:\n\n    auditorium demo\n\nAnd point your browser at [localhost:6789](http://localhost:6789).\n\n## Quick Start and Tutorials\n\nIf you want to quickly grok `auditorium`, the best option is to [look at the demo online](https://auditorium-demo.apiad.net) and then\n[read the source code](https://github.com/apiad/auditorium/blob/master/auditorium/demo.py). This way you will both see the end result and what effort it takes to get there.\n\n* [Authoring a slideshow with Python](https://apiad.net/auditorium/quickstart/#python-first)\n* [Authoring a slideshow with Markdown](https://apiad.net/auditorium/quickstart/#markdown-first)\n* [Rendering a slideshow as purely static HTML](https://apiad.net/auditorium/quickstart/#going-full-static)\n\n## Made with Auditorium\n\n* [Auditorium Demo](https://auditorium-demo.apiad.net) - a slideshow showcasing the most important features from `auditorium`.\n\n### Your Contributions Here\n\nIf you have a slideshow to showcase here, feel free to [edit this Readme](https://github.com/apiad/auditorium/edit/master/README.md) and send a pull request. Add your website, Github repository, and any other information.\n\nIf you feel like sending some support please consider adding a badge somewhere in your website or repository:\n\n```html\n<a href="https://apiad.net/auditorium">\n    <img alt="Made with Auditorium"\n         src="https://img.shields.io/badge/made--with-auditorium-blue">\n    </img>\n</a>\n```\n\nIt looks like this:\n\n<a href="https://apiad.net/auditorium"><img alt="Made with Auditorium" src="https://img.shields.io/badge/made--with-auditorium-blue"></img></a>\n\n## History\n\nSee [the docs](https://apiad.net/auditorium/history).\n\n## Collaboration and License\n\nLicense is MIT, so you know the drill: fork, develop, add tests, pull request, rinse and repeat.\nSee collaboration details [in the docs](https://apiad.net/auditorium/contributing).\n',
    'author': 'Alejandro Piad',
    'author_email': 'alepiad@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
